﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Log;
using Protocols;
using ProtoBuf;
using Redis;
using ServerLib;
using StackExchange.Redis;
using Utility;
using Quartz;
using Quartz.Impl;
using System.Threading;
using System.Collections.Concurrent;

namespace ServerLib
{
    /// <summary>
    /// 作业
    /// </summary>
    public class HelloJob : IJob
    {
        public Task Execute(IJobExecutionContext context)
        {
            // 目前在线程池线程，切换到主线程执行
            //LogSys.Info($"[HelloJob][Execute1],threadid={System.Threading.Thread.CurrentThread.ManagedThreadId}");

            Action callback;
            if (!m_mapCallbacks.TryGetValue(context.JobDetail.Key, out callback))
            {
                return Task.CompletedTask;
            }

            if (null != callback)
            {
                //LogSys.Info($"[HelloJob][Execute2],threadid={System.Threading.Thread.CurrentThread.ManagedThreadId}");
                SendOrPostCallback PostCallback = delegate (Object state)
                {
                    //LogSys.Info($"[HelloJob][Execute3],threadid={System.Threading.Thread.CurrentThread.ManagedThreadId}");
                    callback();
                };

                m_SynchronizationContext.Post(PostCallback, null);
            }
            return Task.CompletedTask;
        }

        public static SingleThreadSynchronizationContext m_SynchronizationContext;
        public static ConcurrentDictionary<JobKey, Action> m_mapCallbacks = new ConcurrentDictionary<JobKey, Action>();
        public static void AddCallback(JobKey key, Action callback)
        {
            m_mapCallbacks.TryAdd(key, callback);
        }

        public static void RemoveCallback(JobKey key)
        {
            Action callback;
            m_mapCallbacks.TryRemove(key, out callback);
        }
    }

    /// <summary>
    /// 作业调度器
    /// </summary>
    public class JobScheduler
    {
        public bool Init()
        {
            //从工厂中获取一个调度器实例化
            scheduler = StdSchedulerFactory.GetDefaultScheduler().Result;

            scheduler.Start();       //开启调度器

            return true;
        }

        public void Destroy()
        {
            foreach (var item in mapJobs)
            {
                DeleteJob(item.Key);
            }
            mapJobs.Clear();
        }

        /// <summary>
        /// 每分钟定时作业, 并且以周期的形式运行
        /// </summary>
        /// <param name="hour"></param>
        /// <param name="minute"></param>
        /// <param name="second"></param>
        /// <param name="callback"></param>
        /// <returns></returns>
        public string AddEveryMinuteJob(int minute, int second, Action callback)
        {
            HelloJob.m_SynchronizationContext = SynchronizationContext.Current as SingleThreadSynchronizationContext;

            //Debug.Assert(minute >= 0 && minute <= 59);
            //Debug.Assert(second >= 0 && second <= 59);

            jobcount++;
            string jobname = jobcount.ToString();
            string triggername = jobcount.ToString();

            IJobDetail job = JobBuilder.Create<HelloJob>()  //创建一个作业
                .WithIdentity(jobname)
                .Build();

            HelloJob.AddCallback(job.Key, callback);

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.AppendFormat("{0} */{1} * * * ? *", second, minute);
            string cronExpression = stringBuilder.ToString();

            ITrigger trigger = TriggerBuilder.Create()
                                        .WithIdentity(triggername)
                                        .WithCronSchedule(cronExpression)
                                        .Build();

            scheduler.ScheduleJob(job, trigger);      //把作业，触发器加入调度器。

            return jobname;
        }

        /// <summary>
        /// 每时定时作业
        /// </summary>
        /// <returns></returns>
        public string AddHourJob(int minute, int second, Action callback)
        {
            HelloJob.m_SynchronizationContext = SynchronizationContext.Current as SingleThreadSynchronizationContext;

            //Debug.Assert(minute >= 0 && minute <= 59);
            //Debug.Assert(second >= 0 && second <= 59);

            jobcount++;
            string jobname = jobcount.ToString();
            string triggername = jobcount.ToString();

            IJobDetail job = JobBuilder.Create<HelloJob>()  //创建一个作业
                .WithIdentity(jobname)
                .Build();

            HelloJob.AddCallback(job.Key, callback);

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.AppendFormat("{0} {1} * * * ? *", second, minute);
            string cronExpression = stringBuilder.ToString();

            ITrigger trigger = TriggerBuilder.Create()
                                        .WithIdentity(triggername)
                                        .WithCronSchedule(cronExpression)
                                        .Build();

            scheduler.ScheduleJob(job, trigger);      //把作业，触发器加入调度器。

            return jobname;
        }

        /// <summary>
        /// 每时定时作业, 并且以周期的形式运行
        /// </summary>
        /// <param name="hour"></param>
        /// <param name="minute"></param>
        /// <param name="second"></param>
        /// <param name="callback"></param>
        /// <returns></returns>
        public string AddEveryHourJob(int hour, int minute, int second, Action callback)
        {
            HelloJob.m_SynchronizationContext = SynchronizationContext.Current as SingleThreadSynchronizationContext;

            //Debug.Assert(minute >= 0 && minute <= 59);
            //Debug.Assert(second >= 0 && second <= 59);

            jobcount++;
            string jobname = jobcount.ToString();
            string triggername = jobcount.ToString();

            IJobDetail job = JobBuilder.Create<HelloJob>()  //创建一个作业
                .WithIdentity(jobname)
                .Build();

            HelloJob.AddCallback(job.Key, callback);

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.AppendFormat("{0} {1} */{2} * * ? *", second, minute, hour);
            string cronExpression = stringBuilder.ToString();

            ITrigger trigger = TriggerBuilder.Create()
                                        .WithIdentity(triggername)
                                        .WithCronSchedule(cronExpression)
                                        .Build();

            scheduler.ScheduleJob(job, trigger);      //把作业，触发器加入调度器。

            return jobname;
        }

        /// <summary>
        /// 每日定时作业
        /// </summary>
        /// <param name="weekday">周几1-7</param>
        /// <param name="hour">小时0-23</param>
        /// <param name="minute">分钟0-59</param>
        /// <param name="second">秒0-59</param>
        /// <param name="callback"></param>
        /// <returns></returns>
        public string AddDayJob(int hour, int minute, int second, Action callback)
        {
            HelloJob.m_SynchronizationContext = SynchronizationContext.Current as SingleThreadSynchronizationContext;

            //Debug.Assert(hour >= 0 && hour <= 23);
            //Debug.Assert(minute >= 0 && minute <= 59);
            //Debug.Assert(second >= 0 && second <= 59);

            jobcount++;
            string jobname = jobcount.ToString();
            string triggername = jobcount.ToString();

            IJobDetail job = JobBuilder.Create<HelloJob>()  //创建一个作业
                .WithIdentity(jobname)
                .Build();

            HelloJob.AddCallback(job.Key, callback);

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.AppendFormat("{0} {1} {2} * * ? *", second, minute, hour);
            string cronExpression = stringBuilder.ToString();

            ITrigger trigger = TriggerBuilder.Create()
                                        .WithIdentity(triggername)
                                        .WithCronSchedule(cronExpression)
                                        .Build();

            scheduler.ScheduleJob(job, trigger);      //把作业，触发器加入调度器。


            return jobname;
        }

        /// <summary>
        /// 每周定时作业
        /// </summary>
        /// <param name="weekday">周几1-7</param>
        /// <param name="hour">小时0-23</param>
        /// <param name="minute">分钟0-59</param>
        /// <param name="second">秒0-59</param>
        /// <param name="callback"></param>
        /// <returns></returns>
        public string AddWeekJob(int weekday, int hour, int minute, int second, Action callback)
        {
            HelloJob.m_SynchronizationContext = SynchronizationContext.Current as SingleThreadSynchronizationContext;

            //Debug.Assert(weekday >= 1 && weekday <= 7);
            //Debug.Assert(hour >= 0 && hour <= 23);
            //Debug.Assert(minute >= 0 && minute <= 59);
            //Debug.Assert(second >= 0 && second <= 59);

            jobcount++;
            string jobname = jobcount.ToString();
            string triggername = jobcount.ToString();

            IJobDetail job = JobBuilder.Create<HelloJob>()  //创建一个作业
                .WithIdentity(jobname)
                .Build();

            HelloJob.AddCallback(job.Key, callback);

            if (weekday == 7)
            {
                weekday = 1;
            }
            else
            {
                weekday = weekday + 1;
            }
            //weekday = weekday - 1;
            //if (weekday == 0)
            //{
            //    weekday = 7;
            //}

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.AppendFormat("{0} {1} {2} ? * {3}", second, minute, hour, weekday);
            string cronExpression = stringBuilder.ToString();

            ITrigger trigger = TriggerBuilder.Create()
                                        .WithIdentity(triggername)
                                        .WithCronSchedule(cronExpression)
                                        .Build();

            scheduler.ScheduleJob(job, trigger);      //把作业，触发器加入调度器。


            return jobname;
        }

        /// <summary>
        /// 每周1号定时作业
        /// </summary>
        /// <param name="hour">小时0-23</param>
        /// <param name="minute">分钟0-59</param>
        /// <param name="second">秒0-59</param>
        /// <param name="callback"></param>
        /// <returns></returns>
        public string AddMonthJob(int hour, int minute, int second, Action callback)
        {
            HelloJob.m_SynchronizationContext = SynchronizationContext.Current as SingleThreadSynchronizationContext;

            //Debug.Assert(hour >= 0 && hour <= 23);
            //Debug.Assert(minute >= 0 && minute <= 59);
            //Debug.Assert(second >= 0 && second <= 59);

            jobcount++;
            string jobname = jobcount.ToString();
            string triggername = jobcount.ToString();

            IJobDetail job = JobBuilder.Create<HelloJob>()  //创建一个作业
                .WithIdentity(jobname)
                .Build();

            HelloJob.AddCallback(job.Key, callback);

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.AppendFormat("{0} {1} {2} 1 * ?", second, minute, hour);
            string cronExpression = stringBuilder.ToString();

            ITrigger trigger = TriggerBuilder.Create()
                                        .WithIdentity(triggername)
                                        .WithCronSchedule(cronExpression)
                                        .Build();

            scheduler.ScheduleJob(job, trigger);      //把作业，触发器加入调度器。


            return jobname;
        }

        /// <summary>
        /// 删除作业
        /// </summary>
        /// <param name="key"></param>
        public void DeleteJob(string name)
        {
            mapJobs.Remove(name);
            JobKey key = new JobKey(name);
            scheduler.DeleteJob(key);

            HelloJob.RemoveCallback(key);
        }

        /// <summary>
        /// 作业组
        /// </summary>
        public int jobcount = 0;

        /// <summary>
        /// 作业调度器
        /// </summary>
        IScheduler scheduler;

        /// <summary>
        /// 所有作业
        /// </summary>
        Dictionary<string, IJobDetail> mapJobs = new Dictionary<string, IJobDetail>();
    }
}
